/**
 * Copyright (C) 2010  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.building;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.net.URL;
import java.util.Date;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.ow2.bonita.util.Base64;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * @author Anthony Birembaut
 *
 */
public class XmlBuilder {
  
  public static final int BYTES_FRAGMENT_LENGTH = 65536;
  
  public static final String BYTES_FRAGMENT_SEPARATOR = "::";
  
  /**
   * Document builder factory
   */
  private DocumentBuilderFactory documentBuilderFactory;
  
  /**
   * Transformer factory
   */
  private TransformerFactory transformerFactory;
  
  /**
   * DOM representation of the XML file to create
   */
  private Document document;
  
  /**
   * the root node
   */
  private Node rootNode;
  
  /**
   * URL of the XSD for validation
   */
  private URL xsdURL;
  
  /**
   * Logger
   */
  private static Logger LOGGER = Logger.getLogger(XmlBuilder.class.getName());
  
  /**
   * Constructor
   */
  public XmlBuilder(DocumentBuilderFactory documentBuilderFactory, TransformerFactory transformerFactory, URL xsdURL) {
    this.documentBuilderFactory = documentBuilderFactory;
    this.transformerFactory= transformerFactory;
    this.xsdURL = xsdURL;
  }

  /**
   * Validate the generated document against the XSD 
   * @throws IOException 
   * @throws SAXException 
   */
  private void validate() throws SAXException, IOException {
      
    if (LOGGER.isLoggable(Level.INFO)) {
      LOGGER.info("perform the validation against the XSD");
    }
    
    SchemaFactory factory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    Schema schemaFile = factory.newSchema(xsdURL);
    
    Validator validator = schemaFile.newValidator();
    validator.validate(new DOMSource(document));
  }
  
  /** 
   * Build a XML form definition file.
   * This is the last method to call once the form has been built.
   * @return a byte array
   * @throws InvalidFormDefinitionException if the generated document is not valid
   * @throws IOException 
   */
  public byte[] done() throws Exception {
    
    document.appendChild(rootNode);
    
    Source source = new DOMSource(document);
    if (xsdURL != null) {
      if (xsdURL.getFile() != null) {
        validate();
      } else {
        LOGGER.severe("unable to find the XSD");
      }
    }

    ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    
    Result resultat = new StreamResult(outputStream);

    Transformer transformer = transformerFactory.newTransformer();
    transformer.setOutputProperty(OutputKeys.INDENT, "yes");
    transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");

    transformer.transform(source, resultat);

    byte[] xmlContent = outputStream.toByteArray();
    outputStream.close();
    
    return xmlContent;
  }
  
  /**
   * Create the document
   * @return the {@link XmlBuilder}
   * @throws ParserConfigurationException 
   */
  public XmlBuilder createDocument() throws ParserConfigurationException {
    
    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
    document = documentBuilder.newDocument();
    document.setXmlVersion("1.0");
    document.setXmlStandalone(true);

    return this;
  }
  
  public Node createRootNode(String name) throws DOMException, IOException, ClassNotFoundException {
    return internalNodeCreation(null, name, null, null);
  }
  
  public Node createRootNode(String name, Map<String, Serializable> attributes) throws DOMException, IOException, ClassNotFoundException {
    return internalNodeCreation(null, name, null, attributes);
  }
  
  public Node createRootNode(String name, Serializable value) throws DOMException, IOException, ClassNotFoundException {
    if (value != null) {
      return internalNodeCreation(null, name, value, null);
    } else {
      return null;
    }
  }
  
  public Node createRootNode(String name, Serializable value, Map<String, Serializable> attributes) throws DOMException, IOException, ClassNotFoundException {
    if (value != null) {
      return internalNodeCreation(null, name, value, attributes);
    } else {
      return null;
    }
  }
  
  public Node createNode(Node parentNode, String name) throws DOMException, IOException, ClassNotFoundException {
    return internalNodeCreation(parentNode, name, null, null);
  }
  
  public Node createNode(Node parentNode, String name, Map<String, Serializable> attributes) throws DOMException, IOException, ClassNotFoundException {
    return internalNodeCreation(parentNode, name, null, attributes);
  }
  
  public Node createNode(Node parentNode, String name, Serializable value) throws DOMException, IOException, ClassNotFoundException {
    if (value != null) {
      return internalNodeCreation(parentNode, name, value, null);
    } else {
      return null;
    }
  }
  
  public Node createNode(Node parentNode, String name, Serializable value, Map<String, Serializable> attributes) throws DOMException, IOException, ClassNotFoundException {
    if (value != null) {
      return internalNodeCreation(parentNode, name, value, attributes);
    } else {
      return null;
    }
  }
  
  public Node internalNodeCreation(Node parentNode, String name, Serializable value, Map<String, Serializable> attributes) throws DOMException, IOException, ClassNotFoundException {
    Element element = document.createElement(name);
    if (value != null) {
      if (value.getClass().isEnum()) {
        element.setTextContent(((Enum<?>)value).name());
      } else if (value instanceof byte[]) {
        element.setTextContent(fragmentAndEncode((byte[])value));
      } else if (value instanceof Date) {
        element.setTextContent(Long.toString(((Date) value).getTime()));
      } else {
        element.setTextContent(value.toString());
      }
    }
    if (attributes != null) {
      for (String attributeName : attributes.keySet()) {
        Serializable attributeValue = attributes.get(attributeName);
        if (attributeValue.getClass().isEnum()) {
          element.setAttribute(attributeName, ((Enum<?>)attributeValue).name());
        } else if (attributeValue instanceof byte[]) {
          element.setAttribute(attributeName, fragmentAndEncode((byte[])attributeValue));
        } else if (attributeValue instanceof Date) {
          element.setAttribute(attributeName, Long.toString(((Date) attributeValue).getTime()));
        } else {
          element.setAttribute(attributeName, attributeValue.toString());
        }
      }
    }
    if (parentNode == null) {
      rootNode = element;
      return element;
    } else {
      return parentNode.appendChild(element);
    }
  }
  
  private String fragmentAndEncode(byte[] bytes) throws IOException {
    
    StringBuilder encodedString = new StringBuilder();
    
    int bytesFragmentLength = BYTES_FRAGMENT_LENGTH;
    
    int nbFragment = bytes.length / bytesFragmentLength;
    int remainingBytes = bytes.length % bytesFragmentLength;
    if (remainingBytes > 0) {
      nbFragment++;
    }
    
    int bytesPosition = 0;
    for(int nbFragmentsProcessed = 0; nbFragmentsProcessed < nbFragment; nbFragmentsProcessed++) {

      ByteArrayOutputStream byteArrayOutputStream = null;
      if (nbFragmentsProcessed + 1 < nbFragment) {
        byteArrayOutputStream = new ByteArrayOutputStream(bytesFragmentLength);
        byteArrayOutputStream.write(bytes, bytesPosition, bytesFragmentLength);
      } else {
        byteArrayOutputStream = new ByteArrayOutputStream(remainingBytes);
        byteArrayOutputStream.write(bytes, bytesPosition, remainingBytes);
      }
      
      String stringFragment = Base64.encodeBytes(byteArrayOutputStream.toByteArray());
      if (bytesPosition != 0) {
        encodedString.append(BYTES_FRAGMENT_SEPARATOR);
      }
      encodedString.append(stringFragment);
      byteArrayOutputStream.close();
      
      bytesPosition+=bytesFragmentLength;
    }
    
    return encodedString.toString();
  }
}
