/**
 * Copyright (C) 2009  BonitaSoft S.A.
 * BonitaSoft, 31 rue Gustave Eiffel - 38000 Grenoble
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade;

import java.util.Set;

import org.ow2.bonita.facade.exception.RoleAlreadyExistsException;
import org.ow2.bonita.facade.exception.RoleNotFoundException;
import org.ow2.bonita.facade.exception.UserAlreadyExistsException;
import org.ow2.bonita.facade.exception.UserNotFoundException;
import org.ow2.bonita.facade.identity.Role;
import org.ow2.bonita.facade.identity.User;

/**
 * Bonita Identity API. 
 * This API purpose is to manage the users referential.
 * @author Anthony Birembaut
 *
 */
public interface IdentityAPI {

  /**
   * Retrieve a user from its username
   * @param username the user username
   * @return a {@link User}
   * @throws UserNotFoundException
   */
  User getUser(String username) throws UserNotFoundException;
  
  /**
   * Retrieve a role from its name
   * @param name the role's name
   * @return a {@link Role}
   * @throws RoleNotFoundException
   */
  Role getRole(String name) throws RoleNotFoundException;
  
  /**
   * Retrieve all the users
   * @return a {@link Set} of {@link User}
   */
  Set<User> getUsers();
  
  /**
   * Retrieve all the roles
   * @return a {@link Set} of {@link Role}
   */
  Set<Role> getRoles();
  
  /**
   * Retrieve a user's roles
   * @param username the user's username
   * @return a {@link Set} of {@link Role}
   * @throws UserNotFoundException
   */
  Set<Role> getUserRoles(String username) throws UserNotFoundException;
  
  /**
   * Retrieve all the users having a given role
   * @param name the user's name
   * @return a {@link Set} of {@link User}
   * @throws RoleNotFoundException
   */
  Set<User> getUsersInRole(String name) throws RoleNotFoundException;
  
  /**
   * Create a new a user
   * @param username the user's username
   * @param password the user's password
   * @return the {@link User} created
   * @throws UserAlreadyExistsException
   */
  User addUser(String username, String password) throws UserAlreadyExistsException;
  
  /**
   * Create a new a user
   * @param username the user's username
   * @param password the user's password
   * @param firstName the user's first name
   * @param lastName the user's last name
   * @param email the user's email
   * @return the {@link User} created
   * @throws UserAlreadyExistsException
   */
  User addUser(String username, String password, String firstName, String lastName, String email) throws UserAlreadyExistsException;
  
  /**
   * Update a user
   * @param oldUsername the actual username of the user to update
   * @param username the user's username
   * @param password the user's password
   * @param firstName the user's first name
   * @param lastName the user's last name
   * @param email the user's email
   * @return the updated {@link User}
   * @throws UserNotFoundException
   * @throws UserAlreadyExistsException
   */
  User updateUser(String oldUsername, String username, String password, String firstName, String lastName, String email) throws UserNotFoundException, UserAlreadyExistsException;
  
  /**
   * Create a new role
   * @param name the role's name
   * @return the {@link Role} created
   * @throws RoleAlreadyExistsException
   */
  Role addRole(String name) throws RoleAlreadyExistsException;
  
  /**
   * Create a new role
   * @param name the role's name
   * @param label the role's label
   * @param description the role's description
   * @return the {@link Role} created
   * @throws RoleAlreadyExistsException
   */
  Role addRole(String name, String label, String description) throws RoleAlreadyExistsException;
  
  /**
   * Update a role
   * @param oldName the actual role name of the role to update
   * @param name the role's name
   * @param label the role's label
   * @param description the role's description
   * @return the updated {@link Role}
   * @throws RoleNotFoundException
   * @throws RoleAlreadyExistsException
   */
  Role updateRole(String oldName, String name, String label, String description) throws RoleNotFoundException, RoleAlreadyExistsException;
  
  /**
   * Add a role to a user
   * @param roleName the role's name
   * @param username the user's username
   * @throws UserNotFoundException
   * @throws RoleNotFoundException
   */
  void addRoleToUser(String roleName, String username) throws UserNotFoundException, RoleNotFoundException;
  
  /**
   * @param username the user's username
   * @param roleNames the user's roles to set
   * @throws UserNotFoundException
   * @throws RoleNotFoundException
   */
  void setUserRoles(String username, Set<String> roleNames) throws UserNotFoundException, RoleNotFoundException;
  
  /**
   * Remove role from a user
   * @param roleName the role's name
   * @param username the user's username
   * @throws UserNotFoundException
   * @throws RoleNotFoundException
   */
  void removeRoleFromUser(String roleName, String username) throws UserNotFoundException, RoleNotFoundException;
  
  /**
   * Permanently remove a user
   * @param username the user's username
   * @throws UserNotFoundException
   */
  void removeUser(String username) throws UserNotFoundException;
  
  /**
   * Permanently remove a role
   * @param name the role's name
   * @throws RoleNotFoundException
   */
  void removeRole(String name) throws RoleNotFoundException;
}
