/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Charles Souillard, Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.BonitaInternalException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.Comment;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.LightActivityInstance;
import org.ow2.bonita.light.LightProcessInstance;
import org.ow2.bonita.light.LightTaskInstance;

/**
 * Getters on workflow recorded data.<br>
 * Operations in this API applies to main entities managed by Bonita such as:<br>
 * processes, process instances, activities, tasks.<br>
 * Returned records are issued from recorded runtime informations (both runtime and archived workflow instances).
 * @author Pierre Vigneras, Charles Souillard
 */
public interface QueryRuntimeAPI {

  public static final String DEFAULT_QUERIER = "queryList";

  /**
   * Returns records for all iterations and multi-instantiations that should append for the given
   * process instance UUID and activity Id.<br>
   * An empty set is returned if no instance is found.
   * @param instanceUUID the instance UUID.
   * @param activityName the activity name.
   * @return the set containing light activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID, String activityName)
  throws InstanceNotFoundException, ActivityNotFoundException;

  /**
   * Gets all process instances. An empty set is returned if no instance is found.
   * @return a set containing all process instances.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstances();

  /**
   * Gets all light process instances. An empty set is returned if no instance is found.
   * @return a set containing all light process instances.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<LightProcessInstance> getLightProcessInstances();

  /**
   * Gets a set of light process instances form their UUIDs.
   * @param instanceUUIDs the collection of instance UUIDs
   * @return a set of light process instances form their UUIDs.
   */
  Set<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs);

  /**
   * Gets a set of light process instances from an index to the page size. This set is a sub-set
   * of getLightProcessInstances.
   * @param fromIndex the index
   * @param pageSize the page size
   * @return a set of light process instances
   */
  List<LightProcessInstance> getLightProcessInstances(int fromIndex, int pageSize);

  /**
   * Gets a set of light process instances from an index to the page size according to the collection
   * of process instance UUIDs. This set is a sub-set of getLightProcessInstances.
   * @param instanceUUIDs the instances UUIDs
   * @param fromIndex the index
   * @param pageSize the page size
   * @return a set of light process instances
   */
  List<LightProcessInstance> getLightProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs, int fromIndex, int pageSize);

  /**
   * Gets a set of light process instances from an index to the page size. This set is a sub-set
   * of getLightProcessInstances and takes only process instances which contain sub-process(es).
   * @param fromIndex the index
   * @param pageSize the page size
   * @return a set of light process instances
   */
  List<LightProcessInstance> getLightParentProcessInstances(int fromIndex, int pageSize);

  /**
   * Gets a list of light task instances which belong to a process instance according to its UUID.
   * @param rootInstanceUUID the process instance UUID
   * @return a list of light task instances.
   */
  List<LightTaskInstance> getLightTaskInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID);

  /**
   * Gets for each process instance UUID the list of light task instances which belong to a process instance according to its UUID.
   * @param rootInstanceUUIDs the process instance UUIDs
   * @return a list of light task instances.
   */
  Map<ProcessInstanceUUID,List<LightTaskInstance>> getLightTaskInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs);

  /**
   * Gets a list of light activity instances which belong to a process instance according to its UUID.
   * @param rootInstanceUUID the process instance UUID
   * @return a list of light activity instances.
   */
  List<LightActivityInstance> getLightActivityInstancesFromRoot(ProcessInstanceUUID rootInstanceUUID);

  /**
   * Gets for each process instance UUID, the list of light activity instances which belong to a process instance according to its UUID.
   * @param rootInstanceUUIDs the process instance UUIDs
   * @return a list of light activity instances.
   */
  Map<ProcessInstanceUUID,List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs);
  
  /**
   * Gets for each process instance UUID, the list of light activity instances which belong to a process instance according to its UUID having the given state.
   * @param rootInstanceUUIDs the process instance UUIDs
   * @return a list of light activity instances.
   */
  Map<ProcessInstanceUUID,List<LightActivityInstance>> getLightActivityInstancesFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, ActivityState state);
  
  /**
   * Gets for each process instance UUID, the light activity instance that have been updated after all others
   * @param rootInstanceUUIDs the process instance UUIDs
   * @return a light activity instance.
   */
  Map<ProcessInstanceUUID, LightActivityInstance> getLightLastUpdatedActivityInstanceFromRoot(Set<ProcessInstanceUUID> rootInstanceUUIDs, boolean considerSystemTaks);

  /**
   * Counts the number of process instances.
   * @return the number of process instances.
   */
  int getNumberOfProcessInstances();

  /**
   * Counts the number of process instances which are not a sub instance.
   * @return the number of parent process instances
   */
  int getNumberOfParentProcessInstances();

  /**
   * Returns all records of instances matching with the given ProcessInstanceUUID.<br>
   * If one of the ProcessInstanceUUID is not found, nothing is added to the result. <br>
   * An empty set is returned if no instance is found.
   * @param instanceUUIDs the instance UUIDs.
   * @return all records of instances matching with the given ProcessInstanceUUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs);
  
  /**
   * Returns all records of instances in one of the given states.
   * @param instanceStates a Collection of the required instance states
   * @return all records of instances in one task in one of the given states.
   */
  Set<ProcessInstance> getProcessInstancesWithInstanceStates(Collection<InstanceState> instanceStates);
  
  /**
   * Returns all records of instances having one task in one of the given states.<br>
   * @param activityStates states
   * @return all records of instances having one task in one of the given states.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstancesWithTaskState(Collection<ActivityState> activityStates);
  
  /**
   * Returns the record of the instance with the given UUID.
   * @param instanceUUID the instance UUID.
   * @return the record of the instance with the given UUID.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Returns the light process instance with the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return the record of the instance with the given UUID.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   */
  LightProcessInstance getLightProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Returns all records of instance for the given process processDefinitionUUID.<br>
   * An empty set is returned if no instance is found.
   * @param processUUID the process definition UUID.
   * @return a set containing all instance records.
   * @throws ProcessNotFoundException if no process has been found with the given process UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException;

  /**
   * Returns the record of the activity with the given activity UUID.
   * @param activityUUID the activity UUID.
   * @return the activity record with the given instance UUID and activity id.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws ActivityNotFoundException if no activity has been found with the given activity id.
   * @throws BonitaInternalException if an exception occurs.
   */
  ActivityInstance getActivityInstance(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException;

  /**
   * Returns all records of activity for the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return the unordered set containing activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Returns the record of the activity with the given activity instance UUID.
   * @param activityInstanceUUID the activity instance UUID.
   * @return the record of the task with the given activity instance UUID.
   * @throws ActivityNotFoundException if no task has been found with the given activity instance UUID.
   */
  LightActivityInstance getLightActivityInstance(ActivityInstanceUUID activityInstanceUUID) throws ActivityNotFoundException;
  
  /**
   * Returns all records of activity for the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return the unordered set containing activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   */
  Set<LightActivityInstance> getLightActivityInstances(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  //all iterations
  /**
   * Returns records for all iterations and multi-instantiations that should append for the given
   * process instance UUID and activity Id.<br>
   * An empty set is returned if no instance is found.
   * @param instanceUUID the instance UUID.
   * @param activityName the activity name.
   * @return the unordered set containing activity records.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<ActivityInstance> getActivityInstances(ProcessInstanceUUID instanceUUID, String activityName)
  throws InstanceNotFoundException, ActivityNotFoundException;

  /**
   * Returns the record of the task with the given task UUID.
   * @param taskUUID the task UUID.
   * @return the record of the task with the given task UUID.
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  TaskInstance getTask(ActivityInstanceUUID taskUUID) throws TaskNotFoundException;

  /**
   * Returns the candidates of the task with the given task UUID.
   * @param taskUUID the task UUID.
   * @return the candidates of the task with the given task UUID.
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<String> getTaskCandidates(final ActivityInstanceUUID taskUUID) throws TaskNotFoundException;
  
  /**
   * Returns the candidates of the tasks with the given task UUID.
   * @param taskUUID the task UUID.
   * @return the candidates of the tasks with the given task UUID.
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Map<ActivityInstanceUUID, Set<String>> getTaskCandidates(final Set<ActivityInstanceUUID> taskUUIDs) throws TaskNotFoundException;
  
  /**
   * Returns the record of the task with the given task UUID.
   * @param taskUUID the task UUID.
   * @return the record of the task with the given task UUID.
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   */
  LightTaskInstance getLightTaskInstance(ActivityInstanceUUID taskUUID) throws TaskNotFoundException;

  /**
   * Returns all records of task for the given process instance UUID.
   * @param instanceUUID the instance UUID.
   * @return a set containing all task records for the instance.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<TaskInstance> getTasks(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * See getTasks(ProcessInstanceUUID instanceUUID)
   */
  Set<LightTaskInstance> getLightTasks(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the given instance and the authenticated user.<br>
   * @param instanceUUID the instance UUID.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.ActivityState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws InstanceNotFoundException if no instance record is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState) throws InstanceNotFoundException;

  /**
   * See getTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState)
   */
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, ActivityState taskState) throws InstanceNotFoundException;
  
  /**
   * Obtains the user tasks depending on the given activity states for the given instance
   * and the authenticated user.<br>
   * @param instanceUUID the instance UUID.
   * @param taskStates the {@link org.ow2.bonita.facade.runtime.ActivityState states} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws InstanceNotFoundException if no instance record is found with the given instance UUID.
   */
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates) throws InstanceNotFoundException;

  /**
   * See getTaskList(ProcessInstanceUUID instanceUUID, Collection taskStates)
   */
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, Collection<ActivityState> taskStates) throws InstanceNotFoundException;
  
  /**
   * Obtains the tasks with state either READY or EXECUTING or SUSPENDED or FINISHED for the given instance
   * and for the given user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.
   * Otherwise all the users that belong to the candidate list can get the task.
   * @param instanceUUID the instance UUID.
   * @param userId the userId for which the tasks are searched.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.ActivityState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws InstanceNotFoundException if no instance record is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<TaskInstance> getTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState) throws InstanceNotFoundException;

  /**
   * See getTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState)
   */
  Collection<LightTaskInstance> getLightTaskList(ProcessInstanceUUID instanceUUID, String userId, ActivityState taskState) throws InstanceNotFoundException;
  
  /**
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the authenticated user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.<br>
   * Otherwise all the users that belong to the candidate list can get the task.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.ActivityState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<TaskInstance> getTaskList(ActivityState taskState);

  /**
   * See getTaskList(ActivityState taskState).
   */
  Collection<LightTaskInstance> getLightTaskList(ActivityState taskState);
  
  /**
   * Gets an activity UUID among all available tasks of the logged user according to the given
   * activity state.
   * @param taskState the activity state
   * @return an activity UUID
   */
  ActivityInstanceUUID getOneTask(ActivityState taskState);

  /**
   * Gets an activity UUID among all available tasks of the logged user according to the given
   * activity state and the given process instance UUID.
   * @param instanceUUID the process instance UUID
   * @param taskState the activity state
   * @return an activity UUID
   */
  ActivityInstanceUUID getOneTask(ProcessInstanceUUID instanceUUID, ActivityState taskState);

  /**
   * Gets an activity UUID among all available tasks of the logged user according to the given
   * activity state and the given process UUID.
   * @param processUUID the process UUID
   * @param taskState the activity state
   * @return an activity UUID
   */
  ActivityInstanceUUID getOneTask(ProcessDefinitionUUID processUUID, ActivityState taskState);

  /**
   * Obtains the user tasks with state either READY or EXECUTING or SUSPENDED or FINISHED
   * for the given user.<br>
   * If the task has been assigned to a user, only this user can get the task into the returned list.<br>
   * Otherwise all the users that belong to the candidate list can get the task.
   * @param userId the userId for which the tasks are searched.
   * @param taskState the {@link org.ow2.bonita.facade.runtime.ActivityState state} of the task.
   * @return a collection of task records. If no tasks are found, an empty collection is returned.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Collection<TaskInstance> getTaskList(String userId, ActivityState taskState);
  
  /**
   * See getTaskList(String userId, ActivityState taskState)
   */
  Collection<LightTaskInstance> getLightTaskList(String userId, ActivityState taskState);

  /**
   * Gets the activity state of the activity according to its UUID
   * @param activityUUID the activity UUID
   * @return the activity state of the activity according to its UUID
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   */
  ActivityState getActivityInstanceState(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException;

  /**
   * Obtains a variable defined as local to the activity for the given activity UUID and variable name.
   * The activity should either be executed or currently pointed by the a process execution.<br>
   * @param activityUUID the activity UUID.
   * @param variableName the variable name.
   * @return the variable object (can be: a plain {@link String}, a {@link Boolean}, a {@link Date}, a {@link Long} or a {@link Double}).
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws VariableNotFoundException if no variable is found with the given name.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID, String variableName)
  throws ActivityNotFoundException, VariableNotFoundException;

  /**
   * Obtains the variables defined as local to the activity for the given activity UUID.<br>
   * An empty map is returned if no variable is found.
   * @param activityUUID the activity UUID.
   * @return the map of activity variables where key is the variable id and value is the variable object
   * (can be: a plain {@link String}, a {@link Boolean}, a {@link Date}, a {@link Long} or a {@link Double})).
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID activityUUID)
  throws ActivityNotFoundException;

  /**
   * Obtains a variable for the given activity and variable name.
   * This variable could be local to the activity or global to the process.
   * The activity should either be executed or currently pointed by the a process execution.
   * @param activityUUID the activity UUID.
   * @param variableName the variable name.
   * @return the variable object (can be: a plain {@link String}, a {@link Boolean}, a {@link Date},
   * a {@link Long} or a {@link Double}).
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws VariableNotFoundException if no variable is found with the given name.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getVariable(ActivityInstanceUUID activityUUID, String variableName)
  throws ActivityNotFoundException, VariableNotFoundException;

  /**
   * Obtains the activity variables (including global process and local activity variables)
   * for the given activity UUID.<br>
   * An empty map is returned if no variable is found.
   * @param activityUUID the activity UUID.
   * @return the map of activity variables where key is the variable id and value is the variable object
   * (can be: a plain {@link String}, a {@link Boolean}, a {@link Date}, a {@link Long} or a {@link Double}).
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   * @throws InstanceNotFoundException if no recorded informations is found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getVariables(ActivityInstanceUUID activityUUID)
  throws InstanceNotFoundException, ActivityNotFoundException;

  /**
   * Obtains a process variable for the given process instance UUID and variable name.
   * @param instanceUUID the instance UUID.
   * @param variableName the variable name.
   * @return the variable object (can be: a plain {@link String}, a {@link Boolean}, a {@link Date},
   * a {@link Long} or a {@link Double}).
   * @throws InstanceNotFoundException if no instance has found with the given instance UUID.
   * @throws VariableNotFoundException if no variable is found with the given name.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID, String variableName)
  throws InstanceNotFoundException, VariableNotFoundException;

  /**
   * Obtains the process variables for the given process instance UUID.
   * An empty map is returned if no process variable is found.
   * @param instanceUUID the instance UUID.
   * @return the map of process variables where key is the variable id and value is the variable object.
   * @throws InstanceNotFoundException if no instance has been found with the given instance UUID.
   * @throws BonitaInternalException if an other exception occurs.
   */
  Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID instanceUUID)
  throws InstanceNotFoundException;

  /**
   * Obtains all the comments (activity and process) of a ProcessInstance.
   * An empty List is returned if the Process has no feed.
   * @param instanceUUID the instance UUID
   * @return the list containing all the comments
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  List<Comment> getCommentFeed(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Obtains the comments of an activity.
   * @param activityUUID the activity UUID
   * @return the list containing activity comments
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  List<Comment> getActivityInstanceCommentFeed(ActivityInstanceUUID activityUUID) throws InstanceNotFoundException;

  /**
   * Obtains the comments belonging to the process.
   * @param instanceUUID the instance UUID
   * @return the list containing the process comments
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  List<Comment> getProcessInstanceCommentFeed(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Counts the number of comments of an activity.
   * @param activityUUID the activity UUID
   * @return the number of comments of an activity
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  int getNumberOfActivityInstanceComments(ActivityInstanceUUID activityUUID) throws InstanceNotFoundException;
  
  /**
   * Counts the number of comments of all given activities.
   * @param activityUUIDs
   * @param queryList
   * @return
   * @throws InstanceNotFoundException
   */
  Map<ActivityInstanceUUID, Integer> getNumberOfActivityInstanceComments(Set<ActivityInstanceUUID> activityUUIDs) throws InstanceNotFoundException;

  /**
   * Counts the number of comments of a process.
   * @param instanceUUID the instance UUID
   * @return the number of comments of a process
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  int getNumberOfProcessInstanceComments(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Counts the number of all comments (activity and process) of a process.
   * @param instanceUUID  the instance UUID
   * @return the number of all comments (activity and process) of a process
   * @throws InstanceNotFoundException if no instance was found with the given instance UUID.
   */
  int getNumberOfComments(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException;

  /**
   * Returns true if the given task is READY and:
   * - if the task is assigned: if the assigned user is the logged user
   * - if the task is not assigned: if the logged user is in the candidates list 
   * @param taskUUID the activity instance UUID of the task
   * @return true if the task can be executed
   * @throws TaskNotFoundException if no task has been found with the given task UUID.
   */
  boolean canExecuteTask(ActivityInstanceUUID taskUUID) throws TaskNotFoundException;

  /**
   * Returns all instances started by the logged user
   * @return all instances started by the logged user
   */
  Set<ProcessInstance> getUserInstances();

  /**
   * Returns all instances started by the logged user
   * @return all instances started by the logged user
   */
  Set<LightProcessInstance> getLightUserInstances();

  /**
   * Returns all records of instance for the given process processDefinitionUUID.<br>
   * An empty set is returned if no instance is found.
   * @param processUUID the process definition UUID.
   * @return a set containing all instance records.
   * @throws ProcessNotFoundException if no process has been found with the given process UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<LightProcessInstance> getLightProcessInstances(ProcessDefinitionUUID processUUID);

  /**
   * Returns all records of instance for the given process processDefinitionUUIDs.<br>
   * An empty set is returned if no instance is found.
   * @param processUUIDs the process definition UUIDs.
   * @return a set containing all instance records.
   * @throws ProcessNotFoundException if no process has been found with a given process UUID.
   * @throws BonitaInternalException if an exception occurs.
   */
  Set<LightProcessInstance> getLightWeightProcessInstances(Set<ProcessDefinitionUUID> processUUIDs);

  /**
   * Obtains the process attachment names.
   * @param instanceUUID the instance UUID
   * @return the attachment names of a process instance
   */
  Set<String> getAttachmentNames(ProcessInstanceUUID instanceUUID);

  /**
   * Obtains for each attachment (given by its name) the its last version for a process instance
   * @param instanceUUID the process instance UUID
   * @param attachmentNames the attachment names
   * @return a collection of {@link AttachmentInstance}. The method getAttachmentValue should be used to retrieve the content of an attachment
   */
  Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, Set<String> attachmentNames);

  /**
   * Obtains the last versions of some process attachments (the attachment name should match with the regular expression)
   * @param instanceUUID the instance UUID
   * @param regex the regular expression
   * @return a collection of {@link AttachmentInstance}. The method getAttachmentValue should be used to retrieve the content of an attachment
   */
  Collection<AttachmentInstance> getLastAttachments(ProcessInstanceUUID instanceUUID, String regex);

  /**
   * Obtains versions of a process attachment according to its UUID and the attachment name.
   * @param instanceUUID the process instance UUID
   * @param attachmentName the attachment name
   * @return a list of {@link AttachmentInstance}. The method getAttachmentValue should be used to retrieve the content of an attachment
   */
  List<AttachmentInstance> getAttachments(ProcessInstanceUUID instanceUUID, String attachmentName);

  /**
   * Obtains the last attachment of a process according to its UUID and the attachment name.
   * @param instanceUUID the process instance UUID
   * @param attachmentName the attachment name
   * @return an {@link AttachmentInstance} corresponding to the last version of the attachment. The method getAttachmentValue should be used to retrieve the content of an attachment
   */
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName);

  /**
   * Obtains the last version of a process attachment created before the given date.
   * @param instanceUUID the process instance UUID
   * @param attachmentName the attachment name
   * @param date the date
   * @return an {@link AttachmentInstance} corresponding to the last version of the attachment created before the date. The method getAttachmentValue should be used to retrieve the content of an attachment
   */
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, Date date);

  /**
   * Obtains the last version of a process attachment created before an activity end.
   * @param instanceUUID the process instance UUID
   * @param attachmentName the attachment name
   * @param activityUUID the activity instance UUID
   * @return an {@link AttachmentInstance} corresponding to the last version of the attachment created before the end of the activity. The method getAttachmentValue should be used to retrieve the content of an attachment
   * @throws ActivityNotFoundException if this activity is not pointed by a process execution
   * and the execution informations for this activity has not been recorded.
   */
  AttachmentInstance getLastAttachment(ProcessInstanceUUID instanceUUID, String attachmentName, ActivityInstanceUUID activityUUID)
  throws ActivityNotFoundException;

  /**
   * Gets the content of an attachment instance as a byte array.
   * This method is the only way to retrieve an attachment content from an attachment instance.
   * @param attachmentInstance the attachment instance
   * @return the content of an attachment instance
   */
  byte[] getAttachmentValue(AttachmentInstance attachmentInstance);

}
