/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.internal;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.UncancellableInstanceException;
import org.ow2.bonita.facade.exception.UndeletableInstanceException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.AttachmentInstance;
import org.ow2.bonita.facade.runtime.InitialAttachment;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.GroovyException;

/**
 * For internal use only.
 */
public interface InternalRuntimeAPI extends Remote {

  ProcessInstanceUUID instantiateProcess(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException, RemoteException;

  ProcessInstanceUUID instantiateProcess(ProcessDefinitionUUID processUUID, Map<String, Object> variables)
  throws ProcessNotFoundException, RemoteException, VariableNotFoundException;
  
  ProcessInstanceUUID instantiateProcess(ProcessDefinitionUUID processUUID, Map<String, Object> variables, Collection<InitialAttachment> attachments)
    throws ProcessNotFoundException, RemoteException, VariableNotFoundException;

  void startTask(ActivityInstanceUUID taskUUID, boolean assignTask) throws TaskNotFoundException, IllegalTaskStateException, RemoteException;

  void finishTask(ActivityInstanceUUID taskUUID, boolean taskAssign) throws TaskNotFoundException, IllegalTaskStateException, RemoteException;

  void executeTask(ActivityInstanceUUID taskUUID, boolean assignTask) throws TaskNotFoundException, IllegalTaskStateException, RemoteException;
  
  void suspendTask(ActivityInstanceUUID taskUUID, boolean assignTask) throws TaskNotFoundException,
    IllegalTaskStateException, RemoteException;

  void resumeTask(ActivityInstanceUUID taskUUID, boolean taskAssign) throws TaskNotFoundException,
    IllegalTaskStateException, RemoteException;

  void setProcessInstanceVariable(ProcessInstanceUUID instanceUUID, String variableId, Object variableValue)
    throws InstanceNotFoundException, RemoteException, VariableNotFoundException;

  void setActivityInstanceVariable(ActivityInstanceUUID activityUUID, String variableId,
      Object variableValue) throws ActivityNotFoundException, VariableNotFoundException, RemoteException;
  
  void cancelProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException,
    UncancellableInstanceException, RemoteException;
  
  void cancelProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs) throws InstanceNotFoundException,
  UncancellableInstanceException, RemoteException;

  void enableEventsInFailure(ProcessInstanceUUID instanceUUID, String activityName) throws RemoteException;
  void enableEventsInFailure(ActivityInstanceUUID activityUUID) throws RemoteException;
  
  void deleteProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException,
    UndeletableInstanceException, RemoteException;

  void deleteProcessInstances(Collection<ProcessInstanceUUID> instanceUUIDs) throws InstanceNotFoundException, 
    UndeletableInstanceException, RemoteException;

  void deleteAllProcessInstances(Collection<ProcessDefinitionUUID> processUUIDs) throws ProcessNotFoundException,
    UndeletableInstanceException, RemoteException;
  
  void deleteAllProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException,
    UndeletableInstanceException, RemoteException;

  void assignTask(ActivityInstanceUUID taskUUID) throws TaskNotFoundException, RemoteException;
  void assignTask(ActivityInstanceUUID taskUUID, String actorId) throws TaskNotFoundException, RemoteException;
  void assignTask(ActivityInstanceUUID taskUUID, java.util.Set<java.lang.String> candidates) throws TaskNotFoundException, RemoteException;
  void unassignTask(ActivityInstanceUUID taskUUID) throws TaskNotFoundException, RemoteException;

  void startActivity(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException, RemoteException;
  
  void setVariable(ActivityInstanceUUID activityUUID, String variableId, Object variableValue)
    throws ActivityNotFoundException, VariableNotFoundException, RemoteException;

  @Deprecated
  void addComment(final ProcessInstanceUUID instanceUUID, ActivityInstanceUUID activityUUID, String message, String userId)
  throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;
  
  void addComment(final ProcessInstanceUUID instanceUUID, final String message, final String userId)
  throws InstanceNotFoundException, RemoteException;
  
  void addComment(final ActivityInstanceUUID activityUUID, final String message, final String userId)
  throws ActivityNotFoundException, InstanceNotFoundException, RemoteException;
  
  void addProcessMetaData(ProcessDefinitionUUID uuid, String key, String value) throws ProcessNotFoundException, RemoteException;
  void deleteProcessMetaData(ProcessDefinitionUUID uuid, String key) throws ProcessNotFoundException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ProcessInstanceUUID instanceUUID, boolean propagate)
    throws InstanceNotFoundException, GroovyException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ProcessInstanceUUID processInstanceUUID, Map<String, Object> context, boolean propagate)
    throws InstanceNotFoundException, GroovyException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ActivityInstanceUUID activityUUID, boolean propagate)
    throws InstanceNotFoundException, ActivityNotFoundException, GroovyException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ActivityInstanceUUID activityInstanceUUID, Map<String, Object> context, boolean propagate)
    throws InstanceNotFoundException, ActivityNotFoundException, GroovyException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ProcessDefinitionUUID processDefinitionUUID)
    throws ProcessNotFoundException, GroovyException, RemoteException;
  
  Object evaluateGroovyExpression(String expression, ProcessDefinitionUUID processDefinitionUUID, Map<String, Object> context)
    throws ProcessNotFoundException, GroovyException, RemoteException;

  void addAttachment(ProcessInstanceUUID instanceUUID, String name, String label, String description, String fileName, Map<String, String> metadata, byte[] value) throws RemoteException;
  void addAttachment(ProcessInstanceUUID instanceUUID, String name, String fileName, byte[] value) throws RemoteException;
  void addAttachments(Map<AttachmentInstance, byte[]> attachments) throws RemoteException;
  
  void setActivityInstancePriority(ActivityInstanceUUID activityInstanceUUID, int priority) throws ActivityNotFoundException, RemoteException;
  
  void deleteEvents(final String eventName, final String toProcessName, final String toActivityName, final ActivityInstanceUUID actiivtyUUID) throws RemoteException;

  Map<String, Object> executeConnector(String connectorClassName, Map<String, Object[]> parameters)
    throws RemoteException, Exception;
  Map<String, Object> executeConnector(String connectorClassName, Map<String, Object[]> parameters, ProcessDefinitionUUID definitionUUID)
    throws RemoteException, Exception;
  Map<String, Object> executeConnector(String connectorClassName, Map<String, Object[]> parameters, ClassLoader classLoader)
  throws RemoteException, Exception;

  Set<String> executeFilter(String connectorClassName, Map<String, Object[]> parameters, Set<String> members)
    throws RemoteException, Exception;
  Set<String> executeFilter(String connectorClassName, Map<String, Object[]> parameters, Set<String> members, ProcessDefinitionUUID definitionUUID)
    throws RemoteException, Exception;
  Set<String> executeFilter(String connectorClassName, Map<String, Object[]> parameters, Set<String> members, ClassLoader classLoader)
    throws RemoteException, Exception;

  Set<String> executeRoleResolver(String connectorClassName, Map<String, Object[]> parameters, ProcessDefinitionUUID definitionUUID)
    throws RemoteException, Exception;
  Set<String> executeRoleResolver(String connectorClassName, Map<String, Object[]> parameters)
    throws RemoteException, Exception;
  Set<String> executeRoleResolver(String connectorClassName, Map<String, Object[]> parameters, ClassLoader classLoader)
    throws RemoteException, Exception;
}
