/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 * 
 * Modified by Matthieu Chaffotte - BonitaSoft S.A.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ActivityState;
import org.ow2.bonita.facade.runtime.AssignUpdate;
import org.ow2.bonita.facade.runtime.StateUpdate;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.Update;
import org.ow2.bonita.facade.runtime.VariableUpdate;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.light.impl.LightActivityInstanceImpl;
import org.ow2.bonita.type.Variable;
import org.ow2.bonita.util.CopyTool;
import org.ow2.bonita.util.ExceptionManager;
import org.ow2.bonita.util.Misc;

/**
 * @author Pierre Vigneras
 */
public class ActivityInstanceImpl extends LightActivityInstanceImpl implements ActivityInstance, TaskInstance {

  private static final long serialVersionUID = -8515098234372896097L;
  protected long dbid;

  protected List<StateUpdate> stateUpdates = new ArrayList<StateUpdate>();
  protected Map<String, Variable> variables;
  protected Map<String, Object> clientVariables;
  protected List<VariableUpdate> variableUpdates = new ArrayList<VariableUpdate>();
  protected List<AssignUpdate> assignUpdates;
  protected Set<String> candidates;

  protected ActivityInstanceImpl() { }

  public ActivityInstanceImpl(final ActivityInstanceUUID uuid, final ActivityDefinition activityDefinition,
      final ProcessDefinitionUUID processUUID, final ProcessInstanceUUID instanceUUID, final ProcessInstanceUUID rootInstanceUUID,
      final String iterationId, final String activityInstanceId) {
    super(uuid, activityDefinition, processUUID, instanceUUID, rootInstanceUUID, iterationId, activityInstanceId);
  }

  public ActivityInstanceImpl(final ActivityInstance src) {
    super(src);
    this.clientVariables = src.getVariablesBeforeStarted();
        
    List<VariableUpdate> list = src.getVariableUpdates();
    if (list != null && !list.isEmpty()) {
      this.variableUpdates = new ArrayList<VariableUpdate>();
      for (VariableUpdate varUpdate : list) {
        this.variableUpdates.add(new VariableUpdateImpl(varUpdate));
      }
    }
    final List<StateUpdate> stateList = src.getStateUpdates();
    if (stateList != null && !stateList.isEmpty()) {
      this.stateUpdates = new ArrayList<StateUpdate>();
      for (final StateUpdate update : stateList) {
        this.stateUpdates.add(new StateUpdateImpl(update));
      }
    }

    if (src.isTask()) {
      TaskInstance task = src.getTask();      
      List<AssignUpdate> assignList = task.getAssignUpdates();
      if (assignList != null && !assignList.isEmpty()) {
        this.assignUpdates = new ArrayList<AssignUpdate>();
        for (AssignUpdate update : assignList) {
          this.assignUpdates.add(new AssignUpdateImpl(update));
        }
      }
      this.candidates = CopyTool.copy(task.getTaskCandidates());
    }
  }

  public String toString() {
    String userId;
    try {
      userId = getTaskUser();
    } catch (IllegalStateException e) {
      userId = null;
    }
    Set<String> candidates;
    try {
      candidates = getTaskCandidates();
    } catch (IllegalStateException e) {
      candidates = null;
    }

    String st = this.getClass().getName()
    + "[uuid: " + getUUID()
    + ", activityId: " + getActivityName()
    + ", iterationId: " + getIterationId()
    + ", processDefinitionUUID: " + getProcessDefinitionUUID()
    + ", processUUID: " + getProcessInstanceUUID()
    + ", variablesBeforeStarted: " + getVariablesBeforeStarted()
    + ", variableUpdates: " + getVariableUpdates()
    + ", startedDate: " + getStartedDate()
    + ", endedDate: " + getEndedDate()
    + ", readyDate: " + getReadyDate()
    + ", userId: " + userId
    + ", candidates: " + candidates
    + ", state: " + getState()
    + ", createdDate: " + getCreatedDate()
    + ", startedBy: " + getStartedBy()
    + ", startedDate: " + getStartedDate()
    + ", endedDate: " + getEndedDate()
    + ", endedBy: " + getEndedBy()
    + "]";
    return st;
  }

  public Map<String, Object> getVariablesBeforeStarted() {
  	if (this.variables != null) {
  		return CopyTool.getVariableValues(this.variables);
  	}
  	return this.clientVariables;
  }

  public Object getVariableValueBeforeStarted(String variableId) {
    return getVariablesBeforeStarted().get(variableId);
  }

  public TaskInstance getTask() {
    if (isTask()) {
      return (TaskInstance) this;
    }
    return null;
  }

  public List<VariableUpdate> getVariableUpdates() {
    return variableUpdates;
  }

  public Map<String, Object> getLastKnownVariableValues() {
    Map<String, Object> var = getVariablesBeforeStarted();
    if (var != null) {
      var = new HashMap<String, Object>(var);
    } else {
      var = new HashMap<String, Object>();
    }
    for (VariableUpdate varUp : getVariableUpdates()) {
      var.put(varUp.getName(), varUp.getValue());
    }
    return var;
  }

  public List<StateUpdate> getStateUpdates() {
    return this.stateUpdates;
  }

  public void setActivityState(final ActivityState newState, final String userId) {
    updateLastUpdateDate();
    ActivityState oldState = getState();
    this.state = newState;
    final Date actual = new Date();
    //add a state update
    this.getStateUpdates().add(new StateUpdateImpl(actual, newState, oldState, userId));
    if (ActivityState.READY.equals(newState)) {
      readyDate = actual.getTime();
    } else if (ActivityState.EXECUTING.equals(newState)) {
      startedDate = actual.getTime();
      startedBy = userId;
    } else if (ActivityState.FINISHED.equals(newState) || ActivityState.CANCELLED.equals(newState)
        || ActivityState.ABORTED.equals(newState)) {
      endedDate = actual.getTime();
      endedBy = userId;
    }
  }

  public void addVariableUpdate(final VariableUpdate varUpdate) {
    updateLastUpdateDate();
    this.variableUpdates.add(varUpdate);
  }

  public String getUpdatedBy() {
    Update lastUpdate = getLastUpdate();
    if (lastUpdate == null) {
      return null;
    }
    if (lastUpdate.getUpdatedBy() != null) {
      return lastUpdate.getUpdatedBy();
    }
    String message = ExceptionManager.getInstance().getFullMessage("baoi_TII_2", this.getUUID());
    throw new IllegalStateException(message);
  }

  public List<AssignUpdate> getAssignUpdates() {
    return assignUpdates;
  }

  public StateUpdate getLastStateUpdate() {
    if (stateUpdates != null && !stateUpdates.isEmpty()) {
      return stateUpdates.get(stateUpdates.size() - 1); 
    }
    return null;
  }

  public AssignUpdate getLastAssignUpdate() {
    if (assignUpdates != null && !assignUpdates.isEmpty()) {
      return assignUpdates.get(assignUpdates.size() - 1); 
    }
    return null;
  }

  protected Update getLastUpdate() {
    Update lastState = null;
    if (stateUpdates != null && !stateUpdates.isEmpty()) { 
      lastState = stateUpdates.get(stateUpdates.size() - 1);
    }

    Update lastAssign = null;
    if (assignUpdates != null && !assignUpdates.isEmpty()) { 
      lastAssign = assignUpdates.get(assignUpdates.size() - 1);
    }

    if (lastState == null && lastAssign == null) {
      return null;
    } else if (lastState == null && lastAssign != null) { 
      return lastAssign;
    } else if (lastState != null && lastAssign == null) {
      return lastState;
    }
    if (lastAssign.getUpdatedDate().getTime() <= lastState.getUpdatedDate().getTime()) {
      return lastState;
    }
    return lastAssign;
  }

  public void setTaskAssign(final ActivityState taskState,
      final String loggedInUserId,
      final String assignedUserId) {
    updateLastUpdateDate();
    if (getAssignUpdates() == null) {
      this.assignUpdates = new ArrayList<AssignUpdate>();
    }
    
    this.userId = assignedUserId;
    
    this.getAssignUpdates().add(new AssignUpdateImpl(new Date(), taskState, loggedInUserId, this.candidates, this.userId));
  }
  
  public void setTaskAssign(final ActivityState taskState,
      final String loggedInUserId,
      final Set<String> candidates) {
    updateLastUpdateDate();
    if (getAssignUpdates() == null) {
      this.assignUpdates = new ArrayList<AssignUpdate>();
    }
    
    this.userId = null;
    this.candidates = candidates;
    
    this.getAssignUpdates().add(new AssignUpdateImpl(new Date(), taskState, loggedInUserId, this.candidates, this.userId));
  }

  protected void updateLastUpdateDate() {
    this.lastUpdate = System.currentTimeMillis(); 
  }

  public void setVariables(final Map<String, Variable> variables) {
    Misc.badStateIfNotNull(this.variables, "variablesBeforeStarted can not be set twice!");
    this.variables = variables;
    this.clientVariables = null;
  }

  public long getDbid() {
    return dbid;
  }

  public void setExpectedEndDate(final Date expectedEndDate) {
    this.expectedEndDate = Misc.getTime(expectedEndDate);
  }

  public void setReadyDate(final Date readyDate) {
    this.readyDate = Misc.getTime(readyDate); 
  }
  
  public void setEndedDate(final Date endedDate) {
    this.endedDate = Misc.getTime(endedDate);
  }
  

  public Set<String> getTaskCandidates() {
    if (this.candidates == null) {
      return Collections.emptySet();
    }
    return this.candidates;
  }

}
